/* Exercise : Vectorization
 * Version	: 1.0 (2013/06/11)
 * Author	: Carlos Rosales-Fernandez (carlos@tacc.utexas.edu)
 * 
 * This is a simple exercise designed to show loops that vectorize 
 * (main loop) and loops that do not (initialization loop, which 
 * includes calls to an external function). This exercise is part 
 * of the Texas Advanced Computing Center training materials for 
 * programming on the MIC (Intel Xeon Phi) platform. 
 */
 
#include <stdio.h>
#include <time.h>
#include <omp.h>
#include <malloc.h>

#define N 256

int main( int argc, char *argv[])
{
	int		i, j, k, seed;
	double 	tstart, tend, testValue;
	double 	*x, *y, *z;

	// Allocate memory aligned to a 64 byte boundary
	x = (double *)memalign(64,N*sizeof(double));
	y = (double *)memalign(64,N*sizeof(double));
	z = (double *)memalign(64,N*sizeof(double));

	// Initialize the source arrays with random values
	seed = 8719;
	srand(seed);
	for( i = 0; i < N; i++) x[i] = 0.e0;
	for( j = 0; j < N; j++) y[j] = rand()/(double)RAND_MAX;
	for( k = 0; k < N; k++) z[k] = rand()/(double)RAND_MAX;

        for( k = 1; k < N-1; k++) y[k] = y[k-1] + y[k+1];
        testValue = y[N-1] + z[N-1];

//#pragma omp parallel
//{
	tstart = omp_get_wtime();
	// Add an external loop to measure a reasonable time interval
	for(i = 0; i < 100000; i++){
		#pragma vector aligned
		for( j = 0; j < N; j++){
			x[j] =  y[j] + z[j];
		}
	}
	tend = omp_get_wtime();
//}

	printf("\n Vectorization exercise completed in %9.5f seconds.", tend-tstart );
	printf("\n Validation: x[N-1] = %6.2f and it should be %6.2f\n\n", (float)x[N-1],(float)testValue);
	
	free( x ); free( y ); free( z );
	return 0;
}
