// ****************************************************************************
// simple async demo/test
// compile with: icc -openmp
//
//    Uncomment the wait pragma to achieve the resynch.
//
//    wait clauses require an explicit device number (mic:0)
//
//    offload_wait does not offload code; offload with a wait clause does
//
//    Arbitrary uninitialized signal var seems to work fine.
//
//    CAUTION: the executable line after the wait gets offloaded!
//
// djames (at) tacc.utexas.edu
// 25 Sep 2013
//
//    09 Jul 2014   Gave tag variable a unique name
//
// ****************************************************************************

#include <stdio.h>
#include <omp.h>
#include <unistd.h>  // gives us sleep function

// ------------------------------------------------------------------

__declspec( target(mic) ) void incrementSlowly( int* pm );

// ------------------------------------------------------------------
int main( void ) {

    int n   = 123;
    int tag = 1;

    double t1, t2, t3, t4;

    printf("CPU says: MIC is doing offloading works.\n");
    t1 = omp_get_wtime();
    #pragma offload target(mic:0)  signal( &tag )  
    {
      printf("MIC says: I am doing offloading works.\n");
      printf( "\t logical cores on mic: %d\n", omp_get_num_procs() );
      printf( "\t maximum threads on mic: %d\n\n", omp_get_max_threads() );
      incrementSlowly( &n );  // CPU will not wait for MIC codes.
    }

    t2 = omp_get_wtime();

//  CPU codes can be written here to do some calculations 
//  at the same time when MIC is busy.
//  If CPU code is finished first, it will wait at the following offload_wait cluase,
//  until the MIC code is finished.
//  If MIC code is finished first, it is just empty automatically,
//  and the CPU code will be excuted in sequence.
    printf("CPU can do some works here when MIC is busy.\n");
    printf(".......\n\n");
    

  #pragma offload_wait target(mic:0)  wait( &tag )
//  #pragma offload target(mic:0)  wait( &tag )
    {
      printf("Works on MIC are finished. This section is done by CPU.\n");
      //printf("Works on MIC are finished. This section is done by MIC\n");
      printf( "\t logical cores: %d\n", omp_get_num_procs() );
      printf( "\t maximum threads: %d\n\n", omp_get_max_threads() );
      //fflush(0);
    }

    t3 = omp_get_wtime();
 
    if ( n == 123 )  {
       printf( "The offload increment has NOT finished...\n" );
    } else {  // n has changed
       printf( "The offload increment finish successfully...\n" );
    }

    printf( "\tn = %d \n\n", n );

    t4 = omp_get_wtime();
    /*printf("\n t2 - t1 = %9.5f seconds.", t2-t1 );
    printf("\n t3 - t2 = %9.5f seconds.", t3-t2 );
    printf("\n t4 - t3 = %9.5f seconds.", t4-t3 );*/

    return 0;

}

// ------------------------------------------------------------------

void incrementSlowly(   int* pm )  {
  (*pm)++;
  sleep(2);
  return;
}
