! *****************************************************************************
! add one-line directive to achieve offload
! compile with: ifort -openmp
!
! NOTE: Fortran compiler is fairly picky about what "one-liners" it allows
! below an offload directive.  In particular, it does NOT allow something
! like "m = 2*n"; the compiler generates the syntax error:
!
!   error #8547: The OFFLOAD directive must be followed by one of the 
!   OpenMP PARALLEL directives, a CALL statement, or an assignment 
!   statement where the right-hand side is only a function invocation.
!
! The C compiler doesn't seem to be so picky.
!
! djames (at) tacc.utexas.edu
! 29 Sep 2023
! *****************************************************************************

program main

  use omp_lib

  integer :: nprocs
  integer :: maxthreads 

  !dir$ offload begin target(mic)
    nprocs     = omp_get_num_procs()
    maxthreads = omp_get_max_threads()
  !dir$ end offload

  print*, "   total procs: ", nprocs
  print*, "   max threads: ", maxthreads

end program
