// ****************************************************************************
// offload clauses with array on the stack
// compile with: icc -openmp
//
// CAUTION: default stacksize is 12MB
//
// djames (at) tacc.utexas.edu
// 26 Sep 2013
//
// Revision History
// ****************
//
//    04 Oct 2013   Removed alloc_if and free_if.  These apparently have
//                  no effect on stack-based arrays.  To see this, deliberately
//                  set alloc_if(0) on all arrays.
//
// ****************************************************************************

#include <stdio.h>
#include <omp.h>

// -------------------------------------------------------------------

int main( void ) {

  double a[100000], b[100000], c[100000], d[100000];
    // on the stack; literal here is important for now
  int i;

  for ( i=0; i<100000; i++ )  {
    a[i] = 1.0;
    b[i] = (double)(i);
  }

  double tstart, tend;
  tstart = omp_get_wtime();

  #pragma offload target(mic:0)        
//   in( a ) out( c, d ) inout( b )
    #pragma omp parallel for
      for ( i=0; i<100000; i++ )  {
        c[i] =  a[i] + b[i];
        d[i] =  a[i] - b[i];
        b[i] = -b[i];
      }

  tend = omp_get_wtime();

  printf("\n Off06 completed in %9.5f seconds.", tend-tstart );

  printf( "\n\t last a val  = %f",     a[100000-1]);
  printf( "\n\t last b val  = %f",     b[100000-1]);
  printf( "\n\t last c val  = %f",     c[100000-1]);
  printf( "\n\t last d val  = %f\n\n", d[100000-1]);

  return 0;

}
