// ****************************************************************************
// offload clauses with data on the heap (dynamic allocations)
// compile with: icc -openmp
//
// Variations:
//
//   Define "const int N=500000" and then "double a[N]" et al without malloc.
//     In this case, the C compiler requires length attribute,
//     but C++ does not.
//
//   Pointers global with declspec: works fine
//
//   Pointers local to main: this works fine too (a bit of a surprise)
//
// djames (at) tacc.utexas.edu
// 29 Sep 2013
// ****************************************************************************

#include <stdio.h>
#include <omp.h>
#include <malloc.h>

// -------------------------------------------------------------------

// __declspec( target(mic) ) double *a, *b;

int main( void ) {

  int i;
  int N = 5000000;
  //int N = 50000000;
  double *a, *b;

  a = ( double* ) memalign( 64, N*sizeof(double) );
  b = ( double* ) memalign( 64, N*sizeof(double) );

  for ( i=0; i<N; i++ )  {
    a[i] = (double)(i);
  }

  #pragma offload target(mic)                   \
   in(  a : length(N) alloc_if(1) free_if(1) ), \
   out( b : length(N) alloc_if(1) free_if(1) )
    #pragma omp parallel for
      for ( i=0; i<N; i++ )  {
        b[i] = 2.0 * a[i];
      }

  printf( "\n\t last a val  = %f",     a[N-1]);
  printf( "\n\t last b val  = %f\n\n", b[N-1]);

  free(a);
  free(b);

  return 0;

}
